@@ -16,8 +16,8 @@ from functools import lru_cache
 from typing import cast, Any, Callable, Dict, List, Iterable, Iterator, \
     MutableMapping, Optional, Set, Union, Tuple, Type
 
-from ..exceptions import XMLSchemaKeyError, XMLSchemaTypeError, XMLSchemaValueError, \
-    XMLSchemaRuntimeError, XMLSchemaWarning
+from ..exceptions import XMLSchemaKeyError, XMLSchemaTypeError, \
+    XMLSchemaValueError, XMLSchemaWarning
 from ..names import XSD_NAMESPACE, XSD_REDEFINE, XSD_OVERRIDE, XSD_NOTATION, \
     XSD_ANY_TYPE, XSD_SIMPLE_TYPE, XSD_COMPLEX_TYPE, XSD_GROUP, \
     XSD_ATTRIBUTE, XSD_ATTRIBUTE_GROUP, XSD_ELEMENT, XSI_TYPE
@@ -190,18 +190,29 @@ class XsdGlobals(XsdValidator):
             XSD_GROUP: validator.xsd_group_class,
             XSD_ELEMENT: validator.xsd_element_class,
         }
+        self.load_namespace = lru_cache(maxsize=1000)(self._load_namespace)
 
     def __repr__(self) -> str:
         return '%s(validator=%r, validation=%r)' % (
             self.__class__.__name__, self.validator, self.validation
         )
 
+    def __getstate__(self) -> Dict[str, Any]:
+        state = self.__dict__.copy()
+        state.pop('load_namespace', None)
+        return state
+
+    def __setstate__(self, state: Dict[str, Any]) -> None:
+        self.__dict__.update(state)
+        self.load_namespace = lru_cache(maxsize=1000)(self._load_namespace)
+
     def copy(self, validator: Optional[SchemaType] = None,
              validation: Optional[str] = None) -> 'XsdGlobals':
         """Makes a copy of the object."""
-        obj = self.__class__(self.validator if validator is None else validator,
-                             validation or self.validation)
-
+        obj = self.__class__(
+            validator=self.validator if validator is None else validator,
+            validation=validation or self.validation
+        )
         obj.namespaces.update(self.namespaces)
         obj.types.update(self.types)
         obj.attributes.update(self.attributes)
@@ -333,7 +344,7 @@ class XsdGlobals(XsdValidator):
             global_map[qname] = obj[0],  # To catch circular builds
             global_map[qname] = component = factory_or_class(elem, schema)
 
-            # Apply redefinitions (changing elem involve a re-parsing of the component)
+            # Apply redefinitions (changing elem involve reparse of the component)
             for elem, schema in obj[1:]:
                 if component.schema.target_namespace != schema.target_namespace:
                     msg = _("redefined schema {!r} has a different targetNamespace")
@@ -469,8 +480,7 @@ class XsdGlobals(XsdValidator):
                          for obj in ns_schemas):
                 ns_schemas.append(schema)
 
-    @lru_cache(maxsize=1000)
-    def load_namespace(self, namespace: str, build: bool = True) -> bool:
+    def _load_namespace(self, namespace: str, build: bool = True) -> bool:
         """
         Load namespace from available location hints. Returns `True` if the namespace
         is already loaded or if the namespace can be loaded from one of the locations,
@@ -583,14 +593,7 @@ class XsdGlobals(XsdValidator):
             # XSD namespace not imported or XSD namespace not managed by a meta-schema.
             # Creates a new meta-schema instance from the XSD meta-schema source and
             # replaces the default meta-schema instance in all registered schemas.
-            if self.validator.meta_schema is None:
-                msg = _("missing default meta-schema instance {!r}")
-                raise XMLSchemaRuntimeError(msg.format(self.validator))
-
-            url = self.validator.meta_schema.url
-            base_schemas = {k: v for k, v in self.validator.meta_schema.BASE_SCHEMAS.items()
-                            if k not in self.namespaces}
-            meta_schema = self.validator.create_meta_schema(url, base_schemas, self)
+            meta_schema = self.validator.create_meta_schema(global_maps=self)
 
             for schema in self.iter_schemas():
                 if schema.meta_schema is not None:
