@@ -25,9 +25,9 @@ from ..names import XSD_NAMESPACE, XSD_ANY_TYPE, XSD_SIMPLE_TYPE, XSD_PATTERN, \
     XSD_LENGTH, XSD_MIN_LENGTH, XSD_MAX_LENGTH, XSD_WHITE_SPACE, XSD_ENUMERATION,\
     XSD_LIST, XSD_ANY_SIMPLE_TYPE, XSD_UNION, XSD_RESTRICTION, XSD_ANNOTATION, \
     XSD_ASSERTION, XSD_ID, XSD_IDREF, XSD_FRACTION_DIGITS, XSD_TOTAL_DIGITS, \
-    XSD_EXPLICIT_TIMEZONE, XSD_ERROR, XSD_ASSERT, XSD_QNAME, XSD_UNTYPED_ATOMIC
+    XSD_EXPLICIT_TIMEZONE, XSD_ERROR, XSD_ASSERT, XSD_QNAME
 from ..translation import gettext as _
-from ..helpers import get_prefixed_qname, local_name
+from ..helpers import local_name
 
 from .exceptions import XMLSchemaValidationError, XMLSchemaEncodeError, \
     XMLSchemaDecodeError, XMLSchemaParseError
@@ -352,16 +352,16 @@ class XsdSimpleType(XsdType, ValidationMixin[Union[str, bytes], DecodedValueType
 
         root_type = self.root_type
         if root_type.name is not None:
-            sequence_type = cast(str, root_type.prefixed_name)
+            sequence_type = f'xs:{root_type.local_name}'
         else:
-            sequence_type = get_prefixed_qname(XSD_UNTYPED_ATOMIC, self.namespaces)
+            sequence_type = 'xs:untypedAtomic'
 
         if not self.is_list():
             return sequence_type
         elif self.is_emptiable():
-            return '{}*'.format(sequence_type)
+            return f'{sequence_type}*'
         else:
-            return '{}+'.format(sequence_type)
+            return f'{sequence_type}+'
 
     def is_empty(self) -> bool:
         return self.max_length == 0 or \
@@ -436,44 +436,45 @@ class XsdSimpleType(XsdType, ValidationMixin[Union[str, bytes], DecodedValueType
 
     def iter_decode(self, obj: Union[str, bytes], validation: str = 'lax',
                     **kwargs: Any) -> IterDecodeType[DecodedValueType]:
-        if obj is None:
-            yield obj
-        else:
-            text = self.normalize(obj)
-            if self.patterns is not None:
-                try:
-                    self.patterns(text)
-                except XMLSchemaValidationError as err:
-                    yield err
+        text = self.normalize(obj)
+        if self.patterns is not None:
+            try:
+                self.patterns(text)
+            except XMLSchemaValidationError as err:
+                yield err
 
-            for validator in self.validators:
-                try:
-                    validator(text)
-                except XMLSchemaValidationError as err:
-                    yield err
+        for validator in self.validators:
+            try:
+                validator(text)
+            except XMLSchemaValidationError as err:
+                yield err
 
-            yield text
+        yield text
 
     def iter_encode(self, obj: Any, validation: str = 'lax', **kwargs: Any) \
             -> IterEncodeType[EncodedValueType]:
-        if not isinstance(obj, (str, bytes)):
-            reason = _("a {0!r} or {1!r} object required").format(str, bytes)
-            yield XMLSchemaEncodeError(self, obj, str, reason)
-        else:
+        if isinstance(obj, (str, bytes)):
             text = self.normalize(obj)
-            if self.patterns is not None:
-                try:
-                    self.patterns(text)
-                except XMLSchemaValidationError as err:
-                    yield err
+        elif obj is None:
+            text = ''
+        elif isinstance(obj, list):
+            text = ' '.join(str(x) for x in obj)
+        else:
+            text = str(obj)
 
-            for validator in self.validators:
-                try:
-                    validator(text)
-                except XMLSchemaValidationError as err:
-                    yield err
+        if self.patterns is not None:
+            try:
+                self.patterns(text)
+            except XMLSchemaValidationError as err:
+                yield err
 
-            yield text
+        for validator in self.validators:
+            try:
+                validator(text)
+            except XMLSchemaValidationError as err:
+                yield err
+
+        yield text
 
     def get_facet(self, tag: str) -> Optional[FacetsValueType]:
         return self.facets.get(tag)
@@ -631,12 +632,11 @@ class XsdAtomicBuiltin(XsdAtomic):
         try:
             result = self.to_python(obj)
         except (ValueError, DecimalException) as err:
-            # todo: maybe need to translate python error messages?
             yield XMLSchemaDecodeError(self, obj, self.to_python, reason=str(err))
             yield None
             return
         except TypeError:
-            # xs:error type (eg. an XSD 1.1 type alternative used to catch invalid values)
+            # xs:error type (e.g. an XSD 1.1 type alternative used to catch invalid values)
             reason = _("invalid value {!r}").format(obj)
             yield self.validation_error(validation, error=reason, obj=obj)
             yield None
@@ -658,7 +658,7 @@ class XsdAtomicBuiltin(XsdAtomic):
                     pass
                 else:
                     try:
-                        result = '{%s}%s' % (kwargs['namespaces'][prefix], name)
+                        result = f"{{{kwargs['namespaces'][prefix]}}}{name}"
                     except (TypeError, KeyError):
                         try:
                             if kwargs['source'].namespace != XSD_NAMESPACE:
@@ -673,7 +673,7 @@ class XsdAtomicBuiltin(XsdAtomic):
                     pass
                 else:
                     if default_namespace:
-                        result = '{%s}%s' % (default_namespace, obj)
+                        result = f'{{{default_namespace}}}{obj}'
 
         elif self.name == XSD_IDREF:
             try:
@@ -1140,7 +1140,7 @@ class XsdUnion(XsdSimpleType):
                     return
                 elif validation == 'strict':
                     # In 'strict' mode avoid lax encoding by similar types
-                    # (eg. float encoded by int)
+                    # (e.g. float encoded by int)
                     break
 
         if hasattr(obj, '__iter__') and not isinstance(obj, (str, bytes)):
@@ -1394,8 +1394,6 @@ class XsdAtomicRestriction(XsdAtomic):
         for result in base_type.iter_decode(obj, validation, **kwargs):
             if isinstance(result, XMLSchemaValidationError):
                 yield result
-                if isinstance(result, XMLSchemaDecodeError):
-                    yield str(obj) if validation == 'skip' else None
             else:
                 if result is not None:
                     for validator in self.validators:
