@@ -478,7 +478,7 @@ class XsdComponent(XsdValidator):
         elif not self._target_namespace:
             self.name = local_name(self.name)
         else:
-            self.name = '{%s}%s' % (self._target_namespace, local_name(self.name))
+            self.name = f'{{{self._target_namespace}}}{local_name(self.name)}'
 
     @property
     def local_name(self) -> Optional[str]:
@@ -507,7 +507,7 @@ class XsdComponent(XsdValidator):
     def build(self) -> None:
         """
         Builds components that are not fully parsed at initialization, like model groups
-        or internal local elements in model groups. Otherwise does nothing.
+        or internal local elements in model groups, otherwise does nothing.
         """
 
     @property
@@ -532,7 +532,7 @@ class XsdComponent(XsdValidator):
         elif not default_namespace:
             return self.name == name or not self.qualified and self.local_name == name
         else:
-            qname = '{%s}%s' % (default_namespace, name)
+            qname = f'{{{default_namespace}}}{name}'
             return self.qualified_name == qname or not self.qualified and self.local_name == name
 
     def match(self, name: Optional[str], default_namespace: Optional[str] = None,
@@ -560,7 +560,7 @@ class XsdComponent(XsdValidator):
 
         # Try a match with other prefixes
         target_namespace = self.target_namespace
-        suffix = ':%s' % self.local_name
+        suffix = f':{self.local_name}'
 
         for k in filter(lambda x: x.endswith(suffix), mapping):
             prefix = k.split(':')[0]
@@ -584,13 +584,13 @@ class XsdComponent(XsdValidator):
         if self.parent is None:
             return self
         component = self.parent
-        while component is not self:  # pragma: no cover
+        while component is not self:
             if component.parent is None:
                 return component
             component = component.parent
-        else:
+        else:  # pragma: no cover
             msg = _("parent circularity from {}")
-            raise XMLSchemaValueError(msg.format(self))  # pragma: no cover
+            raise XMLSchemaValueError(msg.format(self))
 
     def get_parent_type(self) -> Optional['XsdType']:
         """
@@ -609,8 +609,8 @@ class XsdComponent(XsdValidator):
         """
         Creates an iterator for XSD subcomponents.
 
-        :param xsd_classes: provide a class or a tuple of classes to iterates over only a \
-        specific classes of components.
+        :param xsd_classes: provide a class or a tuple of classes to iterate \
+        over only a specific classes of components.
         """
         if xsd_classes is None or isinstance(self, xsd_classes):
             yield self
@@ -618,11 +618,12 @@ class XsdComponent(XsdValidator):
     def iter_ancestors(self, xsd_classes: ComponentClassType = None)\
             -> Iterator['XsdComponent']:
         """
-        Creates an iterator for XSD ancestor components, schema excluded. Stops when the component
-        is global or if the ancestor is not an instance of the specified class/classes.
+        Creates an iterator for XSD ancestor components, schema excluded.
+        Stops when the component is global or if the ancestor is not an
+        instance of the specified class/classes.
 
-        :param xsd_classes: provide a class or a tuple of classes to iterates over only a \
-        specific classes of components.
+        :param xsd_classes: provide a class or a tuple of classes to iterate \
+        over only a specific classes of components.
         """
         ancestor = self
         while True:
@@ -767,6 +768,7 @@ class XsdType(XsdComponent):
     @staticmethod
     def is_complex() -> bool:
         """Returns `True` if the instance is a complexType, `False` otherwise."""
+        raise NotImplementedError()
 
     def is_atomic(self) -> bool:
         """Returns `True` if the instance is an atomic simpleType, `False` otherwise."""
@@ -843,7 +845,7 @@ class XsdType(XsdComponent):
         if self is xsd_type:
             return False
 
-        block = ('%s %s' % (xsd_element.block, xsd_type.block)).strip()
+        block = f'{xsd_element.block} {xsd_type.block}'.strip()
         if not block:
             return False
 
